@@ -28,8 +28,6 @@
 
 from __future__ import annotations
 
-import typing as t
-
 __all__ = (
     "BasicUserActivity",
     "BasicUserActivityUS",
@@ -37,6 +35,7 @@ __all__ = (
     "TimeBasedActivityUS",
     "GeographyBasedActivity",
     "GeographyBasedActivityUS",
+    "GeographyBasedActivityByCity",
     "PlaybackDetailsSubscribedStatus",
     "PlaybackDetailsSubscribedStatusUS",
     "PlaybackDetailsLiveTimeBased",
@@ -76,6 +75,9 @@ __all__ = (
     "AdPerformance",
 )
 
+import logging
+import typing as t
+
 from analytix.abc import DetailedReportType, ReportType
 from analytix.errors import InvalidRequest
 from analytix.reports import data
@@ -92,6 +94,8 @@ from analytix.reports.features import (
     ZeroOrOne,
 )
 
+_log = logging.getLogger(__name__)
+
 
 class BasicUserActivity(ReportType):
     def __init__(self) -> None:
@@ -120,7 +124,9 @@ class BasicUserActivityUS(ReportType):
 class TimeBasedActivity(ReportType):
     def __init__(self) -> None:
         self.name = "Time-based activity"
-        self.dimensions = Dimensions(ExactlyOne("day", "month"))
+        self.dimensions = Dimensions(
+            ExactlyOne("day", "month"), Optional("creatorContentType")
+        )
         self.filters = Filters(
             ZeroOrOne("country", "continent", "subContinent"),
             ZeroOrOne("video", "group"),
@@ -132,7 +138,9 @@ class TimeBasedActivity(ReportType):
 class TimeBasedActivityUS(ReportType):
     def __init__(self) -> None:
         self.name = "Time-based activity (US)"
-        self.dimensions = Dimensions(ExactlyOne("day", "month"))
+        self.dimensions = Dimensions(
+            ExactlyOne("day", "month"), Optional("creatorContentType")
+        )
         self.filters = Filters(
             Required("province"),
             ZeroOrOne("video", "group"),
@@ -144,7 +152,9 @@ class TimeBasedActivityUS(ReportType):
 class GeographyBasedActivity(ReportType):
     def __init__(self) -> None:
         self.name = "Geography-based activity"
-        self.dimensions = Dimensions(Required("country"))
+        self.dimensions = Dimensions(
+            Required("country"), Optional("creatorContentType")
+        )
         self.filters = Filters(
             ZeroOrOne("continent", "subContinent"),
             ZeroOrOne("video", "group"),
@@ -156,7 +166,9 @@ class GeographyBasedActivity(ReportType):
 class GeographyBasedActivityUS(ReportType):
     def __init__(self) -> None:
         self.name = "Geography-based activity (US)"
-        self.dimensions = Dimensions(Required("province"))
+        self.dimensions = Dimensions(
+            Required("province"), Optional("creatorContentType")
+        )
         self.filters = Filters(
             Required("country==US"),
             ZeroOrOne("video", "group"),
@@ -165,11 +177,57 @@ class GeographyBasedActivityUS(ReportType):
         self.sort_options = SortOptions(*self.metrics.values)
 
 
+class GeographyBasedActivityByCity(DetailedReportType):
+    def __init__(self) -> None:
+        self.name = "Geography-based activity (by city)"
+        self.dimensions = Dimensions(
+            Required("city"),
+            ZeroOrMore("creatorContentType", "country", "province", "subscribedStatus"),
+            ZeroOrOne("day", "month"),
+        )
+        self.filters = Filters(
+            ZeroOrOne("country", "province", "continent", "subContinent"),
+            ZeroOrOne("video", "group"),
+        )
+        self.metrics = Metrics(
+            "views",
+            "estimatedMinutesWatched",
+            "averageViewDuration",
+            "averageViewPercentage",
+        )
+        self.sort_options = SortOptions(
+            *data.LOCATION_AND_TRAFFIC_SORT_OPTIONS, descending_only=True
+        )
+        self.max_results = 25
+
+    def validate(
+        self,
+        dimensions: t.Collection[str],
+        filters: dict[str, str],
+        metrics: t.Collection[str],
+        sort_options: t.Collection[str],
+        max_results: int = 0,
+    ) -> None:
+        if 25 < max_results <= 250:
+            _log.warning(
+                "While the documentation says city reports can have a maximum of 250 "
+                "results, the actual maximum the API accepts (currently) is 25"
+            )
+
+        if "province" in dimensions:
+            # Change the filters on the fly to confirm with special
+            # rules for this report type.
+            self.filters = Filters(Required("country==US"), ZeroOrOne("video", "group"))
+
+        super().validate(dimensions, filters, metrics, sort_options, max_results)
+
+
 class PlaybackDetailsSubscribedStatus(ReportType):
     def __init__(self) -> None:
         self.name = "User activity by subscribed status"
         self.dimensions = Dimensions(
-            Optional("subscribedStatus"), ZeroOrOne("day", "month")
+            ZeroOrMore("creatorContentType", "subscribedStatus"),
+            ZeroOrOne("day", "month"),
         )
         self.filters = Filters(
             ZeroOrOne("country", "continent", "subContinent"),
@@ -184,7 +242,8 @@ class PlaybackDetailsSubscribedStatusUS(ReportType):
     def __init__(self) -> None:
         self.name = "User activity by subscribed status (US)"
         self.dimensions = Dimensions(
-            Optional("subscribedStatus"), ZeroOrOne("day", "month")
+            ZeroOrMore("creatorContentType", "subscribedStatus"),
+            ZeroOrOne("day", "month"),
         )
         self.filters = Filters(
             ZeroOrOne("video", "group"),
@@ -198,7 +257,12 @@ class PlaybackDetailsLiveTimeBased(ReportType):
     def __init__(self) -> None:
         self.name = "Time-based playback details (live)"
         self.dimensions = Dimensions(
-            ZeroOrMore("liveOrOnDemand", "subscribedStatus", "youtubeProduct"),
+            ZeroOrMore(
+                "creatorContentType",
+                "liveOrOnDemand",
+                "subscribedStatus",
+                "youtubeProduct",
+            ),
             ZeroOrOne("day", "month"),
         )
         self.filters = Filters(
@@ -214,7 +278,7 @@ class PlaybackDetailsViewPercentageTimeBased(ReportType):
     def __init__(self) -> None:
         self.name = "Time-based playback details (view percentage)"
         self.dimensions = Dimensions(
-            ZeroOrMore("subscribedStatus", "youtubeProduct"),
+            ZeroOrMore("creatorContentType", "subscribedStatus", "youtubeProduct"),
             ZeroOrOne("day", "month"),
         )
         self.filters = Filters(
@@ -231,7 +295,12 @@ class PlaybackDetailsLiveGeographyBased(ReportType):
         self.name = "Geography-based playback details (live)"
         self.dimensions = Dimensions(
             Required("country"),
-            ZeroOrMore("liveOrOnDemand", "subscribedStatus", "youtubeProduct"),
+            ZeroOrMore(
+                "creatorContentType",
+                "liveOrOnDemand",
+                "subscribedStatus",
+                "youtubeProduct",
+            ),
         )
         self.filters = Filters(
             ZeroOrOne("continent", "subContinent"),
@@ -247,7 +316,7 @@ class PlaybackDetailsViewPercentageGeographyBased(ReportType):
         self.name = "Geography-based playback details (view percentage)"
         self.dimensions = Dimensions(
             Required("country"),
-            ZeroOrMore("subscribedStatus", "youtubeProduct"),
+            ZeroOrMore("creatorContentType", "subscribedStatus", "youtubeProduct"),
         )
         self.filters = Filters(
             ZeroOrOne("continent", "subContinent"),
@@ -263,7 +332,12 @@ class PlaybackDetailsLiveGeographyBasedUS(ReportType):
         self.name = "Geography-based playback details (live, US)"
         self.dimensions = Dimensions(
             Required("province"),
-            ZeroOrMore("liveOrOnDemand", "subscribedStatus", "youtubeProduct"),
+            ZeroOrMore(
+                "creatorContentType",
+                "liveOrOnDemand",
+                "subscribedStatus",
+                "youtubeProduct",
+            ),
         )
         self.filters = Filters(
             Required("country==US"),
@@ -279,7 +353,7 @@ class PlaybackDetailsViewPercentageGeographyBasedUS(ReportType):
         self.name = "Geography-based playback details (view percentage, US)"
         self.dimensions = Dimensions(
             Required("province"),
-            ZeroOrMore("subscribedStatus", "youtubeProduct"),
+            ZeroOrMore("creatorContentType", "subscribedStatus", "youtubeProduct"),
         )
         self.filters = Filters(
             Required("country==US"),
@@ -295,7 +369,9 @@ class PlaybackLocation(ReportType):
         self.name = "Playback locations"
         self.dimensions = Dimensions(
             Required("insightPlaybackLocationType"),
-            ZeroOrMore("day", "liveOrOnDemand", "subscribedStatus"),
+            ZeroOrMore(
+                "creatorContentType", "day", "liveOrOnDemand", "subscribedStatus"
+            ),
         )
         self.filters = Filters(
             ZeroOrOne("country", "province", "continent", "subContinent"),
@@ -310,7 +386,7 @@ class PlaybackLocationDetail(DetailedReportType):
     def __init__(self) -> None:
         self.name = "Playback locations (detailed)"
         self.dimensions = Dimensions(
-            Required("insightPlaybackLocationDetail"),
+            Required("insightPlaybackLocationDetail"), Optional("creatorContentType")
         )
         self.filters = Filters(
             Required("insightPlaybackLocationType==EMBEDDED"),
@@ -330,7 +406,9 @@ class TrafficSource(ReportType):
         self.name = "Traffic sources"
         self.dimensions = Dimensions(
             Required("insightTrafficSourceType"),
-            ZeroOrMore("day", "liveOrOnDemand", "subscribedStatus"),
+            ZeroOrMore(
+                "creatorContentType", "day", "liveOrOnDemand", "subscribedStatus"
+            ),
         )
         self.filters = Filters(
             ZeroOrOne("country", "province", "continent", "subContinent"),
@@ -345,7 +423,7 @@ class TrafficSourceDetail(DetailedReportType):
     def __init__(self) -> None:
         self.name = "Traffic sources (detailed)"
         self.dimensions = Dimensions(
-            Required("insightTrafficSourceDetail"),
+            Required("insightTrafficSourceDetail"), Optional("creatorContentType")
         )
         self.filters = Filters(
             Required("insightTrafficSourceType"),
@@ -382,7 +460,13 @@ class DeviceType(ReportType):
         self.name = "Device types"
         self.dimensions = Dimensions(
             Required("deviceType"),
-            ZeroOrMore("day", "liveOrOnDemand", "subscribedStatus", "youtubeProduct"),
+            ZeroOrMore(
+                "creatorContentType",
+                "day",
+                "liveOrOnDemand",
+                "subscribedStatus",
+                "youtubeProduct",
+            ),
         )
         self.filters = Filters(
             ZeroOrOne("country", "province", "continent", "subContinent"),
@@ -403,7 +487,13 @@ class OperatingSystem(ReportType):
         self.name = "Operating systems"
         self.dimensions = Dimensions(
             Required("operatingSystem"),
-            ZeroOrMore("day", "liveOrOnDemand", "subscribedStatus", "youtubeProduct"),
+            ZeroOrMore(
+                "creatorContentType",
+                "day",
+                "liveOrOnDemand",
+                "subscribedStatus",
+                "youtubeProduct",
+            ),
         )
         self.filters = Filters(
             ZeroOrOne("country", "province", "continent", "subContinent"),
@@ -424,7 +514,13 @@ class DeviceTypeAndOperatingSystem(ReportType):
         self.name = "Device types and operating systems"
         self.dimensions = Dimensions(
             Required("deviceType", "operatingSystem"),
-            ZeroOrMore("day", "liveOrOnDemand", "subscribedStatus", "youtubeProduct"),
+            ZeroOrMore(
+                "creatorContentType",
+                "day",
+                "liveOrOnDemand",
+                "subscribedStatus",
+                "youtubeProduct",
+            ),
         )
         self.filters = Filters(
             ZeroOrOne("country", "province", "continent", "subContinent"),
@@ -440,7 +536,7 @@ class ViewerDemographics(ReportType):
         self.name = "Viewer demographics"
         self.dimensions = Dimensions(
             OneOrMore("ageGroup", "gender"),
-            ZeroOrMore("liveOrOnDemand", "subscribedStatus"),
+            ZeroOrMore("creatorContentType", "liveOrOnDemand", "subscribedStatus"),
         )
         self.filters = Filters(
             ZeroOrOne("country", "province", "continent", "subContinent"),
@@ -456,21 +552,23 @@ class EngagementAndContentSharing(ReportType):
         self.name = "Engagement and content sharing"
         self.dimensions = Dimensions(
             Required("sharingService"),
-            Optional("subscribedStatus"),
+            ZeroOrMore("creatorContentType", "subscribedStatus"),
         )
         self.filters = Filters(
             ZeroOrOne("country", "continent", "subContinent"),
             ZeroOrOne("video", "group"),
             Optional("subscribedStatus"),
         )
-        self.metrics = Metrics("viewerPercentage")
+        self.metrics = Metrics("shares")
         self.sort_options = SortOptions(*self.metrics.values)
 
 
 class AudienceRetention(ReportType):
     def __init__(self) -> None:
         self.name = "Audience retention"
-        self.dimensions = Dimensions(Required("elapsedVideoTimeRatio"))
+        self.dimensions = Dimensions(
+            Required("elapsedVideoTimeRatio"), Optional("creatorContentType")
+        )
         self.filters = Filters(
             Required("video"),
             ZeroOrMore("audienceType", "subscribedStatus", "youtubeProduct"),
@@ -499,7 +597,7 @@ class AudienceRetention(ReportType):
 class TopVideosRegional(DetailedReportType):
     def __init__(self) -> None:
         self.name = "Top videos by region"
-        self.dimensions = Dimensions(Required("video"))
+        self.dimensions = Dimensions(Required("video"), Optional("creatorContentType"))
         self.filters = Filters(ZeroOrOne("country", "continent", "subContinent"))
         self.metrics = Metrics(*data.ALL_VIDEO_METRICS)
         self.sort_options = SortOptions(
@@ -511,7 +609,7 @@ class TopVideosRegional(DetailedReportType):
 class TopVideosUS(DetailedReportType):
     def __init__(self) -> None:
         self.name = "Top videos by state"
-        self.dimensions = Dimensions(Required("video"))
+        self.dimensions = Dimensions(Required("video"), Optional("creatorContentType"))
         self.filters = Filters(Required("province"), Optional("subscribedStatus"))
         self.metrics = Metrics(*data.ALL_PROVINCE_METRICS)
         self.sort_options = SortOptions(
@@ -523,7 +621,7 @@ class TopVideosUS(DetailedReportType):
 class TopVideosSubscribed(DetailedReportType):
     def __init__(self) -> None:
         self.name = "Top videos by subscription status"
-        self.dimensions = Dimensions(Required("video"))
+        self.dimensions = Dimensions(Required("video"), Optional("creatorContentType"))
         self.filters = Filters(
             Optional("subscribedStatus"),
             ZeroOrOne("country", "continent", "subContinent"),
@@ -538,7 +636,7 @@ class TopVideosSubscribed(DetailedReportType):
 class TopVideosYouTubeProduct(DetailedReportType):
     def __init__(self) -> None:
         self.name = "Top videos by YouTube product"
-        self.dimensions = Dimensions(Required("video"))
+        self.dimensions = Dimensions(Required("video"), Optional("creatorContentType"))
         self.filters = Filters(
             ZeroOrOne("country", "province", "continent", "subContinent"),
             ZeroOrMore("subscribedStatus", "youtubeProduct"),
@@ -553,7 +651,7 @@ class TopVideosYouTubeProduct(DetailedReportType):
 class TopVideosPlaybackDetail(DetailedReportType):
     def __init__(self) -> None:
         self.name = "Top videos by playback detail"
-        self.dimensions = Dimensions(Required("video"))
+        self.dimensions = Dimensions(Required("video"), Optional("creatorContentType"))
         self.filters = Filters(
             ZeroOrOne("country", "province", "continent", "subContinent"),
             ZeroOrMore("liveOrOnDemand", "subscribedStatus", "youtubeProduct"),
